require 'json'
require 'json-schema'
require 'organization_validator'
require 'transactions_listing_validator'

module QontoOfx
  # Call and verify responses for Qonto API
  class ApiGateway
    def initialize(slug, token)
      @slug = slug
      @token = token
      @organization_validator = OrganizationValidator.new
      @transactions_listing_validator = TransactionsListingValidator.new
    end

    def fetch_organization
      response = RestClient.get(
        "https://thirdparty.qonto.eu/v2/organizations/#{@slug}",
        Authorization: "#{@slug}:#{@token}"
      )
      check_against_validator response, @organization_validator

      JSON.parse(response.body)
    end

    def fetch_transactions(url)
      response = RestClient.get(
        url,
        Authorization: "#{@slug}:#{@token}"
      )

      check_against_validator response, @transactions_listing_validator

      JSON.parse(response.body)
    end

    def check_against_validator(response, validator)
      if response.code != 200
        raise "Qonto API call failed \
          (status: #{response.code}): #{response.body}"
      end

      raise "response does not match schema: \
        #{response.body}" unless validator.valid? response.body
    end
  end
end
